<?xml version='1.0'?>
<!DOCTYPE xsl:stylesheet
[
 <!ENTITY % xslpath  PUBLIC "Stylesheet Paths" "xslpath.ent"> %xslpath;
 %mmlalias;
 %mmlextra;
]>
<!--########################################################################### 
 |    $Id: mathml.presentation.mod.xsl,v 1.5 2001/05/22 00:55:57 ben Exp $      
 |- ###########################################################################
 |   $Author: ben $
 |
 |   This file comes from the DB2LaTeX XSLT Stylesheets by Ramon Casellas who
 |   accepted its reuse and customization for this package.
 |                                                        
 |   PURPOSE: MathML presentation markup.
 |    Note: these elements are not part of the DocBook DTD. I have extended
 |    the docbook DTD in order to support this tags, so that's why I have these 
 |    templates here.
 |   
 |    MathML namespace used -> mml
 + ############################################################################ -->

<xsl:stylesheet version='1.0'
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:mml="http://www.w3.org/1998/Math/MathML" xmlns="http://www.w3.org/1998/Math/MathML">


<!-- MathML text format template -->

<xsl:template name="mml-text-format">
  <xsl:param name="string"/>
  <xsl:call-template name="mml-ent-replace">
    <xsl:with-param name="i">1</xsl:with-param>
    <xsl:with-param name="mapfile" select="document('lyx.mmlent.xml')"/>
    <xsl:with-param name="string" select="$string"/>
  </xsl:call-template>
</xsl:template>


<!-- MathML Entities replacement engine -->

<xsl:template name="mml-ent-replace">
<xsl:param name="i"/>
<xsl:param name="mapfile"/>
<xsl:param name="string"/>
<xsl:choose>
<xsl:when test="($mapfile/lyxmapping/mapping[position()=$i])[1]">
  <xsl:variable name="map" select="($mapfile/lyxmapping/mapping[position()=$i])[1]"/>
  <xsl:call-template name="string-replace">
  <xsl:with-param name="to"><xsl:value-of select="$map/@text"/></xsl:with-param>
  <xsl:with-param name="from"><xsl:value-of select="$map/@key"/>
  </xsl:with-param>
  <xsl:with-param name="string">
    <xsl:call-template name="mml-ent-replace">
      <xsl:with-param name="i" select="$i+1"/>
      <xsl:with-param name="mapfile" select="$mapfile"/>
      <xsl:with-param name="string" select="$string"/>
    </xsl:call-template>
  </xsl:with-param>
  </xsl:call-template>
</xsl:when>
<xsl:otherwise>
  <xsl:value-of select="$string"/>
</xsl:otherwise>
</xsl:choose>
</xsl:template>


<xsl:template match="mml:mrow">
  <xsl:text></xsl:text> <xsl:apply-templates/> <xsl:text></xsl:text>
</xsl:template>


<!-- TOKENS -->
<!-- Math Identifier -->
<xsl:template match="mml:mi">
    <xsl:variable name="fontstyle" select="@fontstyle"/>
    <xsl:variable name="identifier" select="normalize-space(.)"/>
    <xsl:variable name="map">
      <xsl:call-template name="lyx.mapfrom">
        <xsl:with-param name="mapfile" select="document('lyx.mmlent.xml')"/>
        <xsl:with-param name="keyword" select="$identifier"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:choose>
        <xsl:when test="$map!=''">
            <xsl:value-of select="$map"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:copy-of select="$identifier"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

<!-- Math Number -->
<xsl:template match="mml:mn">
  <xsl:copy-of select="normalize-space(.)"/>
</xsl:template>


<!-- Math Operator -->
<xsl:template match="mml:mo">
  <xsl:variable name="operator" select="normalize-space(.)"/>
  <xsl:choose>
  <xsl:when test="$operator='&ApplyFunction;'">
    <xsl:text></xsl:text> </xsl:when>
  <xsl:when test="$operator='&InvisibleComma;'">
    <xsl:text>\, </xsl:text> </xsl:when>
  <xsl:when test="$operator='&InvisibleTimes;'">
    <xsl:text>\, </xsl:text> </xsl:when>
  <xsl:when test="$operator='&Integral;'">
    <xsl:text>\int </xsl:text> </xsl:when>
  <xsl:when test="$operator='&Product;'">
    <xsl:text>\prod</xsl:text> </xsl:when>
  <xsl:when test="$operator='&Sum;'">
    <xsl:text>\sum</xsl:text> </xsl:when>
  <xsl:when test="$operator='&Hat;'">
    <xsl:text>\sphat</xsl:text> </xsl:when>
  <xsl:when test="$operator='&RightArrow;'">
    <xsl:text>\longrightarrow</xsl:text> </xsl:when>
  <xsl:when test="$operator='&DifferentialD;'">
    <xsl:text>d</xsl:text> </xsl:when>
  <xsl:when test="$operator='('">
    <xsl:text> \left( </xsl:text> </xsl:when>
  <xsl:when test="$operator=')'">
    <xsl:text> \right) </xsl:text> </xsl:when>
  <xsl:when test="$operator='{'">
    <xsl:text> \left\{ </xsl:text> </xsl:when>
  <xsl:when test="$operator='}'">
    <xsl:text> \right\} </xsl:text> </xsl:when>
  <xsl:when test="$operator='['">
    <xsl:text> \left[ </xsl:text> </xsl:when>
  <xsl:when test="$operator=']'">
    <xsl:text> \right] </xsl:text> </xsl:when>
  <xsl:when test="$operator='max'">
    <xsl:text> \max </xsl:text> </xsl:when>
  <xsl:when test="$operator='min'">
    <xsl:text> \min </xsl:text> </xsl:when>
  <xsl:when test="$operator='+' or $operator='-' or $operator='/'
                  or $operator='*'">
    <xsl:value-of select="$operator"/>
  </xsl:when>
  <xsl:otherwise>
    <xsl:value-of select="$operator" />
  </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- Math String -->
<xsl:template match="mml:ms">
  <xsl:if test="@lquote">
    <xsl:call-template name="mml-text-format">
      <xsl:with-param name="string" select="@lquote"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:text>\textrm{</xsl:text>
  <xsl:call-template name="mml-text-format">
    <xsl:with-param name="string" select="normalize-space(.)"/>
  </xsl:call-template>
  <xsl:text>}</xsl:text>
  <xsl:if test="@rquote">
    <xsl:call-template name="mml-text-format">
      <xsl:with-param name="string" select="@rquote"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!-- Math Text -->
<xsl:template match="mml:mtext">
  <xsl:text>\textrm{</xsl:text>
  <xsl:call-template name="mml-text-format">
    <xsl:with-param name="string" select="normalize-space(.)"/>
  </xsl:call-template>
  <xsl:text>}</xsl:text>
</xsl:template>

<!-- Math Space -->
<xsl:template match="mml:mspace">
</xsl:template>


<xsl:template match="mml:msup">
  <xsl:apply-templates select="*[1]"/>
  <xsl:text>^{</xsl:text>
  <xsl:apply-templates select="*[2]"/><xsl:text>}</xsl:text>
</xsl:template>

<xsl:template match="mml:msub">
  <xsl:apply-templates select="*[1]"/>
  <xsl:text>_{</xsl:text>
  <xsl:apply-templates select="*[2]"/><xsl:text>}</xsl:text>
</xsl:template>

<xsl:template match="mml:msubsup">
<xsl:choose>
  <xsl:when test="name(*[1])='mo'">
    <xsl:apply-templates select="*[1]"/>
    <!-- sub -->
    <xsl:text>_{</xsl:text>
    <xsl:apply-templates select="*[2]"/><xsl:text>}</xsl:text>
    <!-- super -->
    <xsl:text>^{</xsl:text>
    <xsl:apply-templates select="*[3]"/><xsl:text>}</xsl:text>
  </xsl:when>
  <xsl:otherwise>
    <!-- base -->
    <xsl:text>{</xsl:text>
    <xsl:apply-templates select="*[1]"/><xsl:text>}</xsl:text>
    <!-- sub -->
    <xsl:text>_{</xsl:text>
    <xsl:apply-templates select="*[2]"/><xsl:text>}</xsl:text>
    <!-- super -->
    <xsl:text>^{</xsl:text>
    <xsl:apply-templates select="*[3]"/><xsl:text>}</xsl:text>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="eval-scripts">
  <xsl:param name="pos"/>
  <xsl:param name="which"/>
  <xsl:param name="sub"/>
  <xsl:param name="sup"/>
  <xsl:param name="presub"/>
  <xsl:param name="presup"/>

  <xsl:choose>
  <xsl:when test="$pos &gt; count(*)">
    <xsl:if test="$presub != ''">
      <xsl:text>_{</xsl:text>
      <xsl:value-of select="$presub"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:if test="$presup != ''">
      <xsl:text>^{</xsl:text>
      <xsl:value-of select="$presup"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:apply-templates select="*[1]"/>
    <xsl:if test="$sub != ''">
      <xsl:text>_{</xsl:text>
      <xsl:value-of select="$sub"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:if test="$sup != ''">
      <xsl:text>^{</xsl:text>
      <xsl:value-of select="$sup"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
  </xsl:when>
  <xsl:otherwise>
    <xsl:choose>
    <xsl:when test="name(*[position()=$pos])='mprescripts'">
      <xsl:call-template name="eval-scripts">
        <xsl:with-param name="pos" select="$pos+1"/>
        <xsl:with-param name="sub" select="$sub"/>
        <xsl:with-param name="sup" select="$sup"/>
        <xsl:with-param name="presub" select="$presub"/>
        <xsl:with-param name="presup" select="$presup"/>
        <xsl:with-param name="which" select="'presub'"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <!-- sub | sup | presub | presup -->
      <xsl:variable name="sc">
        <xsl:apply-templates select="*[position()=$pos]"/>
      </xsl:variable>
 
      <xsl:if test="$which='sub'">
        <xsl:call-template name="eval-scripts">
          <xsl:with-param name="pos" select="$pos+1"/>
          <xsl:with-param name="sub" select="concat($sub,$sc)"/>
          <xsl:with-param name="sup" select="$sup"/>
          <xsl:with-param name="presub" select="$presub"/>
          <xsl:with-param name="presup" select="$presup"/>
          <xsl:with-param name="which" select="'sup'"/>
        </xsl:call-template>
      </xsl:if>
      <xsl:if test="$which='sup'">
        <xsl:call-template name="eval-scripts">
          <xsl:with-param name="pos" select="$pos+1"/>
          <xsl:with-param name="sub" select="$sub"/>
          <xsl:with-param name="sup" select="concat($sup,$sc)"/>
          <xsl:with-param name="presub" select="$presub"/>
          <xsl:with-param name="presup" select="$presup"/>
          <xsl:with-param name="which" select="'sub'"/>
        </xsl:call-template>
      </xsl:if>
      <xsl:if test="$which='presub'">
        <xsl:call-template name="eval-scripts">
          <xsl:with-param name="pos" select="$pos+1"/>
          <xsl:with-param name="sub" select="$sub"/>
          <xsl:with-param name="sup" select="$sup"/>
          <xsl:with-param name="presub" select="concat($presub,$sc)"/>
          <xsl:with-param name="presup" select="$presup"/>
          <xsl:with-param name="which" select="'presup'"/>
        </xsl:call-template>
      </xsl:if>
      <xsl:if test="$which='presup'">
        <xsl:call-template name="eval-scripts">
          <xsl:with-param name="pos" select="$pos+1"/>
          <xsl:with-param name="sub" select="$sub"/>
          <xsl:with-param name="sup" select="$sup"/>
          <xsl:with-param name="presub" select="$presub"/>
          <xsl:with-param name="presup" select="concat($presup,$sc)"/>
          <xsl:with-param name="which" select="'presub'"/>
        </xsl:call-template>
      </xsl:if>
  
    </xsl:otherwise>
    </xsl:choose>
  </xsl:otherwise>
  </xsl:choose>
</xsl:template>
  
<xsl:template match="mml:mprescripts">
</xsl:template>

<xsl:template match="mml:none">
<xsl:text>\, </xsl:text>
</xsl:template>

<xsl:template match="mml:mmultiscripts">
  <xsl:call-template name="eval-scripts">
    <xsl:with-param name="pos" select="'2'"/>
    <xsl:with-param name="sub" select="''"/>
    <xsl:with-param name="sup" select="''"/>
    <xsl:with-param name="presub" select="''"/>
    <xsl:with-param name="presup" select="''"/>
    <xsl:with-param name="which" select="'sub'"/>
  </xsl:call-template>
</xsl:template>


<xsl:template match="mml:munder">
<xsl:choose>
  <xsl:when test="normalize-space(*[2]) = '&UnderBar;'">
    <xsl:text>\underline{</xsl:text>
    <xsl:apply-templates select="*[1]"/><xsl:text>}</xsl:text>
  </xsl:when>
  <xsl:when test="normalize-space(*[2]) = '&UnderBrace;'">
    <xsl:text>\underbrace{</xsl:text>
    <xsl:apply-templates select="*[1]"/><xsl:text>}</xsl:text>
  </xsl:when>
  <xsl:otherwise>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>_{</xsl:text>
    <xsl:apply-templates select="*[2]"/>
    <xsl:text>}</xsl:text>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template match="mml:mover">
<xsl:choose>
  <xsl:when test="normalize-space(*[2]) = '&OverBar;'">
    <xsl:text>\overline{</xsl:text>
    <xsl:apply-templates select="*[1]"/><xsl:text>}</xsl:text>
  </xsl:when>
  <xsl:when test="normalize-space(*[2]) = '&#65079;'"> 
    <xsl:text>\overbrace{</xsl:text>
    <xsl:apply-templates select="*[1]"/><xsl:text>}</xsl:text>
  </xsl:when>
  <xsl:when test="normalize-space(*[2]) = '&Hat;'"> 
    <xsl:choose>
    <xsl:when test="@accent='true'">
      <xsl:text>\widehat{</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>\hat{</xsl:text>
    </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates select="*[1]"/><xsl:text>}</xsl:text>
  </xsl:when>
  <xsl:when test="normalize-space(*[2]) = '&#9140;'">
    <xsl:text>\widehat{</xsl:text>
    <xsl:apply-templates select="*[1]"/><xsl:text>}</xsl:text>
  </xsl:when>
  <xsl:otherwise>
    <xsl:apply-templates select="*[1]"/>
    <xsl:text>^{</xsl:text>
    <xsl:apply-templates select="*[2]"/>
    <xsl:text>}</xsl:text>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>


<!-- Math UnderOver -->
<xsl:template match="mml:munderover">
  <xsl:apply-templates select="*[1]"/>
  <xsl:text>^{</xsl:text>
  <xsl:apply-templates select="*[3]"/>
  <xsl:text>}_{</xsl:text>
  <xsl:apply-templates select="*[2]"/>
  <xsl:text>}</xsl:text>
</xsl:template>


<!-- Math Fenced -->
<xsl:template match="mml:mfenced">
<!-- get open,close, separators att -->
  <xsl:variable name="seplength" select="string-length(@separators)"/>

  <xsl:choose>
    <xsl:when test="@open='('"> <xsl:text> \left( </xsl:text> </xsl:when>
    <xsl:when test="@open='{'"> <xsl:text> \left\{ </xsl:text> </xsl:when>
    <xsl:when test="@open='['"> <xsl:text> \left[ </xsl:text> </xsl:when>
    <xsl:otherwise> <xsl:text> \left( </xsl:text> </xsl:otherwise>
  </xsl:choose>

  <xsl:for-each select="*">
    <xsl:apply-templates select="(.)"/>
    <xsl:if test="position()!=last()">
      <xsl:choose>
        <xsl:when test="$seplength=0">
          <xsl:text>,</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
          <xsl:when test="position() &gt; $seplength">
            <xsl:value-of select="substring(../@separators,$seplength,1)"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="substring(../@separators,position(),1)"/>
          </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:for-each>

  <xsl:choose>
    <xsl:when test="@close=')'"> <xsl:text> \right) </xsl:text> </xsl:when>
    <xsl:when test="@close='}'"> <xsl:text> \right\} </xsl:text> </xsl:when>
    <xsl:when test="@close=']'"> <xsl:text> \right] </xsl:text> </xsl:when>
    <xsl:otherwise> <xsl:text> \right) </xsl:text> </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!-- Math frac -->
<xsl:template match="mml:mfrac">
<xsl:choose>
<xsl:when test="@linethickness">
<!-- The linethickness attribute is not supported by LyX 

  <xsl:choose>
    <xsl:when test="@linethickness='thin'">
      <xsl:text> \frac[1pt]{ </xsl:text>
    </xsl:when>
    <xsl:when test="@linethickness='medium'">
      <xsl:text> \frac[1.1pt]{ </xsl:text>
    </xsl:when>
    <xsl:when test="@linethickness='thick'">
      <xsl:text> \frac[1.2pt]{ </xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text> \frac[</xsl:text>
      <xsl:value-of select="@linethickness"/><xsl:text>]{ </xsl:text>
    </xsl:otherwise>
  </xsl:choose>

-->
  <xsl:text> \frac{</xsl:text>
</xsl:when>
<xsl:otherwise>
  <xsl:text> \frac{</xsl:text>
</xsl:otherwise>
</xsl:choose>
<xsl:apply-templates select="*[1]"/>
<xsl:text>}{</xsl:text>
<xsl:apply-templates select="*[2]"/>
<xsl:text>}</xsl:text>
</xsl:template>


<!-- Math msqrt -->
<xsl:template match="mml:msqrt">
  <xsl:text> \sqrt{ </xsl:text>
  <xsl:apply-templates/>
  <xsl:text> }</xsl:text>
</xsl:template>


<!-- Math mroot -->
<xsl:template match="mml:mroot">
  <xsl:text> \sqrt[</xsl:text><xsl:apply-templates select="*[2]"/>
  <xsl:text>]{</xsl:text>
  <xsl:apply-templates select="*[1]"/><xsl:text>}</xsl:text>
</xsl:template>


<!-- Math Table stuff -->

<!-- Gives the number of columns of the mtable -->
<xsl:template name="mtable.format.tabular">
  <xsl:param name="rows"/>
  <xsl:param name="i" select="1"/>
  <xsl:param name="j" select="1"/>
  <xsl:choose>
    <!-- There are still columns to count -->
    <xsl:when test="$rows[$i]/mml:mtd[$j]">
      <xsl:text>c</xsl:text>
      <!-- Recursive for next column -->
      <xsl:call-template name="mtable.format.tabular">
        <xsl:with-param name="rows" select="$rows"/>
        <xsl:with-param name="i" select="$i"/>
        <xsl:with-param name="j" select="$j+1"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="rowno" select="count($rows)"/>
      <xsl:if test="$i &lt; $rowno">
        <!-- Recursive for next line -->
        <xsl:call-template name="mtable.format.tabular">
          <xsl:with-param name="rows" select="$rows"/>
          <xsl:with-param name="i" select="$i+1"/>
          <xsl:with-param name="j" select="$j"/>
        </xsl:call-template>
      </xsl:if>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="mml:mtable">
  <xsl:choose>
  <xsl:when test="count(mml:mlabeledtr)=0">
    <xsl:variable name="rows" select="mml:mtr"/>
    <xsl:text>&#10;\begin{array}{</xsl:text>
    <xsl:call-template name="mtable.format.tabular">
      <xsl:with-param name="rows" select="mml:mtr"/>
    </xsl:call-template>
    <xsl:text>}</xsl:text>
  </xsl:when>
  <xsl:when test="count(mml:mlabeledtr) &gt; 1">
    <xsl:text>\begin{eqnarray}&#10;</xsl:text>
  </xsl:when>
  <xsl:otherwise>
    <xsl:text>\begin{equation}&#10;</xsl:text>
  </xsl:otherwise>
  </xsl:choose>

  <xsl:apply-templates/>

  <xsl:choose>
  <xsl:when test="count(mml:mlabeledtr)=0">
    <xsl:text>\end{array}&#10;</xsl:text>
  </xsl:when>
  <xsl:when test="count(mml:mlabeledtr) &gt; 1">
    <xsl:text>\end{eqnarray}&#10;</xsl:text>
  </xsl:when>
  <xsl:otherwise>
    <xsl:text>\end{equation}&#10;</xsl:text>
  </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="mml:mtable/mml:mlabeledtr">
  <!-- The reference (first child) is lost since it is automatically done -->
  <!-- The label is taken from the id attribute -->
  <xsl:if test="@id">
    <xsl:text>&#10;\label{</xsl:text>
    <xsl:value-of select="normalize-space(@id)"/>
    <xsl:text>}&#10;</xsl:text>
  </xsl:if>
  <xsl:apply-templates select="*[position() &gt; 1]"/>
</xsl:template>

<xsl:template match="mml:mtr">
<!-- Row starts here -->
<xsl:apply-templates/>
<!-- End Row here -->
</xsl:template>

<xsl:template match="mml:mtd">
  <xsl:apply-templates/><xsl:text> &amp;</xsl:text> 
</xsl:template>

<xsl:template match="mml:mtd[position()=last()]">
  <xsl:apply-templates/><xsl:text>\\</xsl:text>
</xsl:template>

<xsl:template match="mml:mtr[position()=last()]/mml:mtd[position()=last()]">
  <xsl:apply-templates/>
</xsl:template>


<!-- Math Phantom -->
<!-- What should we do with mphantom? -->
<xsl:template match="mml:mphantom">
  <xsl:apply-templates mode="phantom"/>
</xsl:template>

<xsl:template match="*" mode="phantom">
</xsl:template>


<!-- Only notation=radical is really managed -->
<xsl:template match="mml:menclose">
  <xsl:choose>
  <xsl:when test="@notation='radical'">
    <xsl:text>\sqrt{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:when>
  <xsl:when test="@notation='longdiv'">
    <xsl:apply-templates/>
  </xsl:when>
  <xsl:when test="@notation='actuarial'">
    <xsl:apply-templates/>
  </xsl:when>
  </xsl:choose>
</xsl:template>


<!-- Unsupported elements -->
<xsl:template match="mml:mstyle|mml:maligngroup|mml:mpadded|mml:malignmark|
                     mml:merror">
  <xsl:apply-templates/>
</xsl:template>

</xsl:stylesheet>
